-----------------------------------------------------------------------------------------------
-- Client Lua Script for RefreshEmpower
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "AbilityBook"
 
-----------------------------------------------------------------------------------------------
-- RefreshEmpower Module Definition
-----------------------------------------------------------------------------------------------
local RefreshEmpower = {} 
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function RefreshEmpower:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.switchTimer = 0;
	self.timerSpeed = 0.1;
	self.triggerTime = 0.2;
	self.active = 1;
	self.activeToggle = 1;
	self.playerLoaded = false;
	self.abilitiesShown = false;
	self.showAlert = true;
	
    return o
end

function RefreshEmpower:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
  Apollo.RegisterSlashCommand("re", "onSlash", self)
end

-----------------------------------------------------------------------------------------------
-- RefreshEmpower OnLoad
-----------------------------------------------------------------------------------------------
function RefreshEmpower:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("RefreshEmpower.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- RefreshEmpower OnDocLoaded
-----------------------------------------------------------------------------------------------
function RefreshEmpower:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register hwandlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)


		-- Do additional Addon initialization here
		self.mainTimer = ApolloTimer.Create(0.5, true, "onMainTimer", self);
		carbineAbilities = Apollo.GetAddon("Abilities");
	end
end

-----------------------------------------------------------------------------------------------
-- RefreshEmpower Functions
-----------------------------------------------------------------------------------------------
--Save variables
function RefreshEmpower:OnSave(eLevel)
  if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  local tSavedData = { }
  
  tSavedData = {
    active = self.activeToggle,
	alert = self.showAlert
  }

  return tSavedData
end

--When we load in
function RefreshEmpower:OnRestore(eType, savedData)
		self.activeToggle = savedData.active;
		self.showAlert = savedData.alert;
end

--Slash handling
function RefreshEmpower:onSlash(cmd, arg)
	--No arguments, give list
	if arg == "" then
		Print("[RefreshEmpower] No Arguments Given");
		Print("[RefreshEmpower] Use '/re' with the following:");
		Print("[RefreshEmpower] toggle - Toggles the addon");
		Print("[RefreshEmpower] togglealert - Toggles the alert");
	end
	
	--Toggle the addon
	if arg:lower() == "toggle" then
		if GameLib.GetPlayerUnit():GetClassId() ~= 4 then
			Print("[RefreshEmpower] You are not a Medic");
		else
			Print("[RefreshEmpower] Toggling.");
			if self.activeToggle == 1 then
				self.activeToggle = 0;
			else
				self.activeToggle = 1;
			end
		end
	end
	
	--Toggle alerts
	if arg:lower() == "togglealert" then
		Print("[RefreshEmpower] Toggling Alert.");
		if self.showAlert == true then
			self.showAlert = false;
		else
			self.showAlert = true;
		end
	end
end

--Main check timer
function RefreshEmpower:onMainTimer()
	if self.activeToggle == 1 then
		--Check if player unit is loaded
		if GameLib.GetPlayerUnit() then
			--Check class
			if self.playerLoaded == false then
				if GameLib.GetPlayerUnit():GetClassId() ~= 4 then
					Print("Medic not detected, unloading RefreshEmpower");
					self.active = 0;
					self.mainTimer:Stop();
					self.playerLoaded = true;
					return
				else
					self.player = GameLib.GetPlayerUnit();
					self.playerLoaded = true;
				end
			end
		
			
			--Check for abilities window open
			if carbineAbilities.tWndRefs.wndMain and carbineAbilities.tWndRefs.wndMain:IsValid() and carbineAbilities.tWndRefs.wndMain:IsShown() then
				self.abilitiesShown = true;
			else
				self.abilitiesShown = false;
			end
			
			--Check on load if we're in combat
			if self.player:IsInCombat() then 
				self.mainTimer:Stop();
			end
			
			--If we are not in combat, mounted, or dead!
			if not self.player:IsInCombat() and not self.player:IsMounted() and not self.player:IsDead() and not self.abilitiesShown then
				if not findBuff(self.player, "Empowering Aura") then
					self.player = GameLib.GetPlayerUnit();
					AbilityBook.NextSpec()
					self.switchbackAlarm = ApolloTimer.Create(self.timerSpeed, true, "onSwitchAlarm", self);
					self.switchbackAlarm:Start();
					self.switchTimer = 0;
				end
			else
				if self.switchbackAlarm then
					self.switchbackAlarm:Stop();
				end
			end
		end
	else
	
	end
end

--Switch alarm
function RefreshEmpower:onSwitchAlarm()
	--Increment our switch timer
	self.switchTimer = self.switchTimer + self.timerSpeed;
	
	--If the time is right
	if self.switchTimer == self.triggerTime then
		if self.showAlert == true then
			Print("Refreshing for Empowering Aura");
		end
		AbilityBook.PrevSpec();
		self.switchbackAlarm:Stop();
	end
end

--Finds a buff, returns true or false
function findBuff(unit, buffName)
	if unit:GetBuffs() then
		local n=0;
		local buffTable = unit:GetBuffs();
		local buffName = buffName;
		for k,v in pairs(buffTable) do
			n=n+1
			if buffTable.arBeneficial[n] then
				if buffTable.arBeneficial[n].splEffect:GetName() == buffName then
					return true;
				end
			end
		end
	end
	
	--I have failed you!
	return false;
end

--Combat logic handling
function RefreshEmpower:OnEnteredCombat(unit, inCombat)
	--Check if we're active
	if self.active == 1 then
		--If it's the player and yes
		if unit == self.player and inCombat then
			--We've entered combat, no need to check for Emp
			self.mainTimer:Stop();
		else if unit == self.player and not inCombat then
			--We've left combat
			self.mainTimer:Start();
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- RefreshEmpower Instance
-----------------------------------------------------------------------------------------------
local RefreshEmpowerInst = RefreshEmpower:new()
RefreshEmpowerInst:Init()
